package com.winhc.repal.mq.consumer;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.winhc.repal.mq.MQConfig;
import com.winhc.repal.mq.MQTag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Properties;

/**
 * @author Aaron
 * @date 2020/12/14 11:41
 * @description
 */
@Slf4j
@Component
public class GradeConsumer {
    @Autowired
    private MQConfig mqConfig;
    @Autowired
    private GradeListener gradeListener;

    @PostConstruct
    public void init() {
        log.info("初始化队列bean begin");
        new Thread(this::initConsumer).start();
        log.info("初始化队列bean end");
    }

    private void initConsumer() {
        log.info("开始初始化队列消费");
        System.setProperty("rocketmq.client.log.loadconfig","false");
        Properties consumerProperties = new Properties();
        consumerProperties.setProperty(PropertyKeyConst.GROUP_ID, mqConfig.getRepalConsumerId());
        consumerProperties.setProperty(PropertyKeyConst.AccessKey, mqConfig.getAccessKey());
        consumerProperties.setProperty(PropertyKeyConst.SecretKey, mqConfig.getSecretKey());
        consumerProperties.setProperty(PropertyKeyConst.ONSAddr, mqConfig.getUrl());
        consumerProperties.setProperty(PropertyKeyConst.ConsumeThreadNums,"2");
        consumerProperties.setProperty(PropertyKeyConst.ConsumeTimeout,"15");
        try {
            Consumer consumer = ONSFactory.createConsumer(consumerProperties);
            consumer.subscribe(mqConfig.getRepalTopic(), MQTag.BizTag.tag_repal.name(), gradeListener);
            consumer.start();
            log.info("初始化队列消费完成");
        } catch (Exception e) {
            log.error("初始化失败", e);
        }
    }
}
