package com.winhc.repal.service.impl;

import com.winhc.repal.entity.UserInfo;
import com.winhc.repal.enums.MsgKindEnum;
import com.winhc.repal.enums.MsgNameEnum;
import com.winhc.repal.mq.bean.SendMsgBean;
import com.winhc.repal.mq.producer.MessageSendService;
import com.winhc.repal.repository.UserInfoMapper;
import com.winhc.repal.service.MessageNoticeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Service("messageNoticeService")
@Slf4j
public class MessageNoticeServiceImpl implements MessageNoticeService {

    @Resource
    private MessageSendService messageSendService;

    @Resource
    private UserInfoMapper userInfoMapper;

    @Override
    public String sendSms(String mobileNo, MsgNameEnum name, Map<String, String> param) {
        log.info("短信推送, mobileNo:" + mobileNo + ", biz:" + name.name());
        SendMsgBean sendMsgBean = new SendMsgBean();
        sendMsgBean.setMessageChannel(MsgKindEnum.sms.name());
        sendMsgBean.setBizName(name.name());
        sendMsgBean.setParam(param);
        sendMsgBean.setSendTarget(mobileNo);
        messageSendService.send(sendMsgBean);
        return "SUCCESS";
    }

    @Override
    public String saveRepalMsg(Long userId, MsgNameEnum name, Map<String, String> param) {
        log.info("站内信推送, userId:" + userId + ", caseId:" + name.name());

        if (userId != null) {
            UserInfo userInfo = userInfoMapper.selectById(userId);
            if (userInfo == null) {
                log.warn("userinfo不存在, userId:" + userId);
                return "FAIL";
            }
            SendMsgBean sendMsgBean = new SendMsgBean();
            sendMsgBean.setMessageChannel(MsgKindEnum.repal.name());
            sendMsgBean.setBizName(name.name());
            sendMsgBean.setParam(param);
            sendMsgBean.setSendTarget(userInfo.getMobileNo());
            sendMsgBean.setUserId(userId);
            sendMsgBean.setExtParam(new HashMap<>());
            messageSendService.send(sendMsgBean);
        }
        return "SUCCESS";
    }
}
