package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import com.winhc.repal.entity.RepalBill;
import com.winhc.repal.entity.RepalBook;
import com.winhc.repal.entity.RepalChangeRecord;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.enums.PermissionEnum;
import com.winhc.repal.enums.RepalBillStatusEnum;
import com.winhc.repal.model.bo.BookDynamicCountBO;
import com.winhc.repal.model.bo.HomeReceivableBO;
import com.winhc.repal.model.dto.AddRepalBookDTO;
import com.winhc.repal.model.dto.DeleteRepalBookDTO;
import com.winhc.repal.model.dto.HomeTypeDTO;
import com.winhc.repal.model.dto.RepalBookNameDTO;
import com.winhc.repal.model.vo.*;
import com.winhc.repal.repository.RepalBookMapper;
import com.winhc.repal.service.*;
import com.winhc.repal.util.BillGradeUtil;
import com.winhc.repal.util.RemindTypeUtil;
import com.winhc.repal.util.UserContextUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description RepalBook 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalBookServiceImpl extends ServiceImpl<RepalBookMapper, RepalBook> implements RepalBookService {

    @Autowired
    private RepalBookMapper repalBookMapper;

    @Autowired
    private RepalBillService repalBillService;

    @Autowired
    private RepalRolePermissionService repalRolePermissionService;

    @Autowired
    private RepalGroupMemberService repalGroupMemberService;

    @Autowired
    private RepalRemindHistoryService repalRemindHistoryService;

    @Autowired
    private RepalRemindReadInfoService repalRemindReadInfoService;

    @Autowired
    private RepalResponsiblePersonService repalResponsiblePersonService;

    @Autowired
    private RepalChangeRecordService repalChangeRecordService;

    @Override
    public HomeReceivableVO getHomeReceivable(HomeTypeDTO dto) {
        HomeReceivableVO vo = new HomeReceivableVO();
        boolean allFlag = repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        // 所有
        LocalDate start = null;
        LocalDate end = null;
        if (Integer.valueOf(1).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = now.minusDays(7);
            end = now;
        }
        if (Integer.valueOf(2).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = now.minusDays(30);
            end = now;
        }
        HomeReceivableBO receivableBO = repalBillService.getBillTotal(null, allFlag ? null : UserContextUtil.getUser().getUserId(), dto.getRepalBookId(), dto.getRepalBillType(), start, end);
        HomeReceivableBO overdueBO = repalBillService.getBillTotal(RepalBillStatusEnum.OVERDUE.getCode(), allFlag ? null : UserContextUtil.getUser().getUserId(), dto.getRepalBookId(), dto.getRepalBillType(), start, end);
        vo.setBillMoneyStr(receivableBO.getBillMoney().setScale(2).toString());
        vo.setBillCount(receivableBO.getBillCount());
        vo.setOverdueTotalStr(overdueBO.getOverdueTotal().setScale(2).toString());
        vo.setOverdueTotalCount(overdueBO.getOverdueTotalCount());
        vo.setTotalBillMoneyStr(receivableBO.getTotalBillMoney().setScale(2).toString());
        vo.setReceiptMoneyStr(receivableBO.getReceiptMoney().setScale(2).toPlainString());
        // 金额不为0计算逾期率
        if (BigDecimal.ZERO.compareTo(receivableBO.getTotalBillMoney()) != 0 && BigDecimal.ZERO.compareTo(overdueBO.getOverdueTotal()) != 0) {
            vo.setOverdueRate(overdueBO.getOverdueTotal().multiply(new BigDecimal(100)).divide(receivableBO.getTotalBillMoney(), 2, RoundingMode.HALF_UP).toString() + "%");
        }
        vo.setVisible(repalGroupMemberService.getMemberVisible(dto.getRepalBillType()));
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteRepalBook(DeleteRepalBookDTO dto) {
        long groupId = repalGroupMemberService.getUserGroupId(UserContextUtil.getUser().getUserId());
        int count = this.count(Wrappers.lambdaQuery(RepalBook.class)
                .eq(RepalBook::getRepalGroupId, groupId)
                .eq(RepalBook::getDeleted, DeletedStatusEnum.NORMAL.getCode()));
        if (count < 2) {
            throw new CommonException(CodeMsg.FAILED, "必须保留一个账本哦～");
        }
        if (!this.update(Wrappers.lambdaUpdate(RepalBook.class)
                .eq(RepalBook::getId, dto.getRepalBookId())
                .set(RepalBook::getDeleted, DeletedStatusEnum.DELETE.getCode()))) {
            throw new CommonException(CodeMsg.FAILED, "参数错误");
        }
        // 账款和账款关联责任人也删掉
        repalBillService.deleteRepalBillByBookId(dto.getRepalBookId());
        // 删除账本回款记录
        repalChangeRecordService.delReturnedByBookId(dto.getRepalBookId());
        return true;
    }

    @Override
    public List<RepalBookInfoVO> getRepalBookInfoList() {
        Set<String> codeSet = repalRolePermissionService.getUserPermissionCodeSet(UserContextUtil.getUser().getUserId());
        boolean receivableFlag = codeSet.contains(PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        boolean receiptFlag = codeSet.contains(PermissionEnum.QUERY_ALL_RECEIPT.getCode());
        return repalBookMapper.getRepalBookInfoList(UserContextUtil.getUser().getUserId(), repalGroupMemberService.getUserGroupId(UserContextUtil.getUser().getUserId()), receivableFlag, receiptFlag);
    }

    @Override
    public List<List<Integer>> getRepalBookDynamic(HomeTypeDTO dto) {
        List<List<Integer>> resultList = new ArrayList<>();
        long userId = UserContextUtil.getUser().getUserId();
        boolean allFlag = repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        // 所有
        LocalDateTime start = null;
        LocalDateTime end = null;
        if (Integer.valueOf(1).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(7), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        if (Integer.valueOf(2).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(30), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        List<BookDynamicCountBO> countList = repalRemindHistoryService.getBookDynamicCount(dto.getRepalBookId(), allFlag ? null : userId, start, end);
        List<Integer> totalCountList = RemindTypeUtil.getCountList(CollUtil.isEmpty(countList) ? null : countList.stream().collect(Collectors.toMap(BookDynamicCountBO::getRemindType, BookDynamicCountBO::getTotalCount)));
        resultList.add(totalCountList);
        // 未读， 有可能是账本下所有未读
        List<Long> billIdList = repalBillService.getBillIdListByBookId(dto.getRepalBookId(), allFlag ? null : userId);;
        RepalBillRemindUnReadVO unReadVO = repalRemindHistoryService.getRemindUnReadVO(billIdList.stream().map(t -> new RepalBill().setId(t).setRepalBookId(dto.getRepalBookId())).collect(Collectors.toList()), userId, start, end);
        List<Integer> unreadList = new ArrayList<Integer>(){{
            add(unReadVO.getUnreadRiskCount());
            add(unReadVO.getUnreadFinanceCount());
            add(unReadVO.getUnreadRankCount());
            add(unReadVO.getUnreadOverdueCount());
        }};
        resultList.add(unreadList);
        return resultList;
    }

    @Override
    public List<HomeGradeVO> getGradeRateList(HomeTypeDTO dto) {
        boolean allFlag = repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        // 所有
        LocalDateTime start = null;
        LocalDateTime end = null;
        if (Integer.valueOf(1).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(7), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        if (Integer.valueOf(2).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(30), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        List<String> gradeList = repalBillService.getBillGradeByBookId(dto.getRepalBookId(), allFlag ? null : UserContextUtil.getUser().getUserId(), start, end);
        if (CollUtil.isEmpty(gradeList)) {
            return new ArrayList<>();
        }
        return BillGradeUtil.getGradeList(gradeList);
    }

    @Override
    public Boolean setBookVisible(Integer type) {
        repalGroupMemberService.updateVisible(type);
        return true;
    }

    @Override
    public Long addRepalBook(AddRepalBookDTO dto) {
        long groupId = repalGroupMemberService.getUserGroupId(UserContextUtil.getUser().getUserId());
        // 新建账本
        RepalBook book = this.getOne(Wrappers.lambdaQuery(RepalBook.class)
                .eq(RepalBook::getUserId, UserContextUtil.getUser().getUserId())
                .eq(RepalBook::getRepalBookName, dto.getRepalBookName())
                .eq(RepalBook::getRepalGroupId, groupId));
        if (Objects.nonNull(book)) {
            throw new CommonException(CodeMsg.FAILED, "账本名称重复");
        }
        RepalBook repalBook = new RepalBook();
        repalBook.setRepalGroupId(groupId);
        repalBook.setRepalBookName(dto.getRepalBookName());
        repalBook.setUserId(UserContextUtil.getUser().getUserId());
        this.save(repalBook);
        return repalBook.getId();
    }

    @Override
    public List<ReturnedMoneyVO> getGraph(HomeTypeDTO dto) {
        boolean allFlag = repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        long userId = UserContextUtil.getUser().getUserId();
        LocalDate start = null;
        LocalDate end = null;
        LocalDate now = LocalDate.now();
        if (Integer.valueOf(1).equals(dto.getQueryType())) {
            start = now.minusDays(7);
            end =now;
        }
        if (Integer.valueOf(2).equals(dto.getQueryType())) {
            start = now.minusDays(30);
            end = now;
        }
        List<RepalChangeRecord> recordList = repalChangeRecordService.getHomeRecord(allFlag ? null : userId, dto.getRepalBookId(), start, end);
        if (CollUtil.isEmpty(recordList)) {
            return new ArrayList<>();
        }
        Map<LocalDate, List<RepalChangeRecord>> map = recordList.stream().collect(Collectors.groupingBy(RepalChangeRecord::getRecordDate));
        return map.entrySet().stream().map(t -> {
            ReturnedMoneyVO vo = new ReturnedMoneyVO();
            vo.setDate(t.getKey());
            vo.setMoneyStr(t.getValue().stream().map(RepalChangeRecord::getTransMoney).reduce(BigDecimal.ZERO, BigDecimal::add).setScale(2).toString());
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public Boolean updateRepalBookName(RepalBookNameDTO dto) {
        this.update(Wrappers.lambdaUpdate(RepalBook.class)
                .eq(RepalBook::getId, dto.getRepalBookId())
                .set(RepalBook::getRepalBookName, dto.getRepalBookName()));
        return true;
    }
}
